; SketchyLISP Library
; Copyright (C) 2005 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; nexpt

; ---conformance---
; SketchyLISP Core

; ---purpose---
; Compute
; .V x
; raised to the power of
; .V y.
; Both
; .V x
; and
; .V y
; must be natural numbers.

; ---args---
; X - list (base)
; Y - list (exponent)

; ---keywords---
; NEXPT function, base, exponent, power

; ---see-also---
; digits, expt, gcd

; ---example---
; (nexpt 3 3) => 27

(define nexpt #t)

(require "digits.scm")
(require "zerop.scm") ; zero?
(require "evenp.scm") ; even?
(require "ntimes.scm") ; n*
(require "nquotient.scm")

; ---model---
; (define (nexpt x y)
;   (cond ((zero? y) 1)
;     (#t (n* x (nexpt x (n- y 1))))))

; ---code---
(define (nexpt x y)
  (letrec
    ((square (lambda (x) (n* x x)))
    (_nexpt (lambda (y)
      (cond ((zero? y) 1)
        ((even? y)
          (square (nexpt x (nquotient y 2))))
        (#t (n* x (square (nexpt x (nquotient y 2)))))))))
    (_nexpt (natural y))))

