; SketchyLISP Library
; Copyright (C) 2005 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; reduce-r

; ---conformance---
; SketchyLISP Core

; ---purpose---
; Iterate through a list. Combine the second-to-last
; member of the list with the last member, the result
; with the third-to-last member, etc. When the given
; list is empty, return a default.

; ---args---
; F - combining function
; A - list to reduce
; DEFAULT - expr to return when A=()

; ---keywords---
; REDUCE-R function, iteration, right-associative,  right-folding

; ---see-also---
; map, reduce

; ---example---
; (reduce-r cons '(a b c d) #f) => (a b c . d)

(define reduce-r #t)

(require "reverse.scm")

; ---code---
(define (reduce-r f a default)
  (letrec
    ((_reduce (lambda (a res)
      (cond ((null? a) res)
        (#t (_reduce (cdr a)
              (f (car a) res)))))))
    (cond ((null? a) default)
      (#t (let ((ra (reverse a)))
            (_reduce (cdr ra) (car ra)))))))
