; SketchyLISP Library
; Copyright (C) 2005 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; reduce

; ---conformance---
; SketchyLISP Core

; ---purpose---
; Iterate through a list. Combine the first member
; of the list with the second member, the result
; with the third member, etc. When the given list
; is empty, return a default.

; ---args---
; F - combining function
; A - list to reduce
; DEFAULT - expr to return when A=()

; ---keywords---
; REDUCE function, iteration, folding

; ---see-also---
; map, reduce-r

; ---example---
; (reduce cons '(a b c d) #f) => (((a . b) . c) . d)

(define reduce #t)

; ---code---
(define (reduce f a default)
  (letrec
    ((_reduce (lambda (a res)
      (cond ((null? a) res)
        (#t (_reduce (cdr a)
              (f res (car a))))))))
    (cond ((null? a) default)
      (#t (_reduce (cdr a) (car a))))))
