; SketchyLISP Example Program
; Copyright (C) 2005,2006 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; iota

; ---conformance---
; R5RS

; ---purpose---
; Generate sequences of natural numbers.

; ---args---
; l - lower limit of sequence
; h - upper limit of sequence

; ---keywords---
; IOTA function, sequence of numbers, arithmetics, natural

; ---example---
; (iota 1 10) => (1 2 3 4 5 6 7 8 9 10)

; (require "lib/digits.scm")
; (require "lib/list.scm")
; (require "lib/append.scm")
; (require "lib/natural.scm")
; (require "lib/equal.scm") ; =
; (require "lib/plus.scm") ; +
; (require "lib/minus.scm") ; -

; ---model---
; (define (iota l h)
;   (cond ((= l h) (list l))
;     (#t (cons l (iota (+ l 1) h)))))

; ---code---
(define (iota l h)
  (letrec
    ((j (lambda (x r)
      (cond ((= x l) (cons l r))
        (#t (j (- x 1) (cons x r)))))))
    (cond ((> l h)
        (bottom '(bad range)))
      (#t (j h '())))))

