; SketchyLISP Example Program
; Copyright (C) 2005 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; ordered?

; ---conformance---
; R5RS

; ---purpose---
; Check whether the members of a list
; are in a given monotonic order.

; ---args---
; X - list
; P - predicate defining order

; ---keywords---
; ORDERED? function, ordered, ordering, relation
; lists, predicate

; ---example---
; (ordered? '(9 8 7 6 5) >) => #t

; (require "lib/caar.scm") ; cadr

; ---model---
; (define (ordered? x p)
;   (cond ((null? x) #t)
;     ((null? (cdr x)) #t)
;     ((p (car x) (cadr x)) (orderedp (cdr x) p))
;     (#t #f)))

; ---code---
(define (ordered? x p)
  (letrec
    ((orderedp (lambda (x)
      (cond ((null? x) #t)
        ((null? (cdr x)) #t)
        ((p (car x) (cadr x))
          (orderedp (cdr x)))
        (#t #f)))))
    (orderedp x)))

